@@ -16,8 +16,11 @@ import music21 as m21
 from functools import reduce
 import fractions as fr
 from math import gcd
+from .utils.Remove import Remove
+from .utils.flatten import flatten
 
 class PCSet:
+    """Pitch-class set abstraction supporting common set-theoretic operations in arbitrary TET systems."""
 
     def __init__(self,pcs,TET=12,UNI=True,ORD=True):
         '''
@@ -89,13 +92,13 @@ class PCSet:
         return(np.unique((self.pcs*t)%self.TET//1).astype(int))
         
     def multiplyBoulez(self,b):
-        # Boulez pitch class multiplication of a x b
+        """Apply Boulez pitch-class multiplication by ``b`` and return the resulting normal order."""
         ivec = self.LISVector()
         m = []
         for i in range(ivec.shape[0]-1):
             mm = (b+ivec[i])%self.TET
             m.append(mm.tolist())
-        return(PCSet(Remove(flatten(m+b)),TET).normalOrder())
+        return(PCSet(Remove(flatten(m+b)),self.TET).normalOrder())
     
     def zeroOrder(self):
         '''
@@ -109,7 +112,7 @@ class PCSet:
         '''
         return((pivot-self.pcs)%self.TET)
 
-    def I(self):
+    def I0(self):
         '''
         •	inverse operation: (-pcs modulo TET)
         '''
@@ -119,7 +122,7 @@ class PCSet:
         '''
         •	most compact normal 0 order between pcs and its inverse
         '''
-        s_orig = self.pcs
+        s_orig = self.pcs.copy()
         sn = np.sum((self.normalOrder()-self.normalOrder()[0])%self.TET)
         self.pcs = self.I()
         si = np.sum((self.normalOrder()-self.normalOrder()[0])%self.TET)
@@ -149,7 +152,7 @@ class PCSet:
         return(np.histogram(itv,bins)[0])
 
     def Op(self,name):
-        # operate on the pcs with a generic distance operator
+        """Operate on the set with the distance operator encoded by ``name``."""
         from .utils.Remove import Remove
         
         def plusAndMinusPermutations(items):
@@ -181,7 +184,7 @@ class PCSet:
         return(Remove(outset))
         
     def VLOp(self,name):
-        # operate on the pcs with a normal-ordered voice-leading operator
+        """Operate on the set with the voice-leading operator encoded by ``name``."""
         
         op = []
         for num in re.findall("[-\d]+", name):
@@ -209,7 +212,7 @@ class PCSet:
         if self.TET != 12:
             print('Forte class defined only for 12-TET')
             return()
-        Fname = m21.chord.Chord(self.primeForm().pcs.tolist()).forteClass
+        Fname = m21.chord.Chord(self.primeForm().tolist()).forteClass
         return(Fname) 
            
     def commonName(self):
@@ -260,6 +263,7 @@ class PCSet:
             return(c)
 
 class PCSetR:
+    """Pitch-class set wrapper that preserves numpy arrays during transformations and comparisons."""
 
     def __init__(self,pcs,TET=12,UNI=True,ORD=True):
         '''
@@ -285,7 +289,7 @@ class PCSetR:
         
         # trivial sets
         if len(self.pcs) == 1:
-            return(PCSetR(self.pcs-self.pcs[0],TET=self.TET))
+            return(PCSetR(self.pcs-self.pcs[0],TET=self.TET,ORD=False,UNI=False))
 #        if len(self.pcs) == 2:
 #            return(PCSetR(self.pcs,TET=self.TET))
 
@@ -311,7 +315,7 @@ class PCSetR:
                 pcs_norm = np.roll(self.pcs,nroll)
                 break
         if np.array(np.where(dist == dist.min())).shape[1] != 1: pcs_norm = self.pcs
-        return(PCSetR(pcs_norm,TET=self.TET))
+        return(PCSetR(pcs_norm,TET=self.TET,ORD=False,UNI=False))
 
     def normal0Order(self):
         '''
@@ -332,7 +336,7 @@ class PCSetR:
         return(PCSetR(np.unique((self.pcs*t)%self.TET//1,TET=self.TET).astype(int)))
         
     def multiplyBoulez(self,b):
-        # Boulez pitch class multiplication of a x b
+        """Apply Boulez pitch-class multiplication by ``b`` returning a new set."""
         ivec = self.LISVector()
         m = []
         for i in range(ivec.shape[0]-1):
@@ -350,23 +354,23 @@ class PCSetR:
         '''
         •	I operation: (-pcs modulo TET)
         '''
-        return(PCSetR((pivot-self.pcs)%self.TET,TET=self.TET))
+        return(PCSetR((pivot-self.pcs)%self.TET,TET=self.TET,ORD=False,UNI=False))
 
     def primeForm(self):
         '''
         •	most compact normal 0 order between pcs and its I
         '''
-        s_orig = self.pcs
+        s_orig = self.pcs.copy()
         sn = np.sum((self.normalOrder().pcs-self.normalOrder().pcs[0])%self.TET)
         self.pcs = self.I().pcs
         si = np.sum((self.normalOrder().pcs-self.normalOrder().pcs[0])%self.TET)
         if sn <= si:
             self.pcs = s_orig
-            return(PCSetR((self.normalOrder().pcs-self.normalOrder().pcs[0])%self.TET,TET=self.TET))
+            return(PCSetR((self.normalOrder().pcs-self.normalOrder().pcs[0])%self.TET,TET=self.TET,ORD=False,UNI=False))
         else:
             tmp = (self.normalOrder().pcs-self.normalOrder().pcs[0])%self.TET
             self.pcs = s_orig
-            return(PCSetR(tmp,TET=self.TET))
+            return(PCSetR(tmp,TET=self.TET,ORD=False,UNI=False))
 
     def intervalVector(self):
         '''
@@ -386,7 +390,7 @@ class PCSetR:
         return(np.histogram(itv,bins)[0])
 
     def Op(self,name):
-        # operate on the pcs with a distance operator O({x})
+        """Operate on the set with the distance operator ``O`` described by ``name``."""
         
         def plusAndMinusPermutations(items):
             for p in iter.permutations(items):
@@ -419,7 +423,7 @@ class PCSetR:
         return(PCSetR(Remove(outset),TET=self.TET))
         
     def VLOp(self,name,norm=True):
-        # operate on the pcs with a (normal-ordered )relational operator R({x})
+        """Apply the voice-leading operator ``R`` to the set, optionally normalising first."""
         op = []
         for num in re.findall("[-\d]+", name):
             op.append(int(num))
@@ -432,7 +436,7 @@ class PCSetR:
             return(PCSetR(selfto,TET=self.TET))
     
     def NROp(self,ops=None):
-        # operate on the triad with a Neo-Rienmannian Operator
+        """Apply a Neo-Riemannian operator (P, L, or R) to the set."""
         x = self.pcs[1]-self.pcs[0]
         y = self.pcs[2]-self.pcs[1]
         if ops == None:
@@ -446,7 +450,7 @@ class PCSetR:
             return(PCSetR((2*x+y-self.pcs)%self.TET,TET=self.TET))
 
     def opsNameVL(self,b,norm=True):
-        # given two vectors returns the name of the (normal-ordered) voice-leading operator R that connects them
+        """Return the name of the minimal voice-leading operator relating ``self`` to ``b``."""
         if norm:
             a = self.normalOrder().pcs
             b = b.normalOrder().pcs  
@@ -475,8 +479,7 @@ class PCSetR:
         return('R('+np.array2string(diff,separator=',').replace(" ","").replace("[","").replace("]","")+')')
 
     def opsNameO(self,b):
-        # given two vectors returns the name of the distance 
-        # operator O that connects them
+        """Return the distance operator name that maps ``self`` onto ``b``."""
         a = np.sort(self.pcs)
         b = np.sort(b.pcs)
         d = np.zeros((b.shape[0]),dtype=int) 
@@ -532,6 +535,7 @@ class PCSetR:
         return(Fname)
 
 class PCmidiR:
+    """Container for MIDI pitch collections with utilities for set-theoretic and contrapuntal operations."""
     
     def __init__(self,midi,UNI=False,ORD=False,TET=12):
         '''
@@ -638,7 +642,7 @@ class PCmidiR:
         •	I operation, including voice-leading preserving contextual inversion
         '''
         if not isinstance(p,list):
-            return(PCmidiR(p-self.midi+p))
+            return(PCmidiR(p-self.midi))
         else:
             # fixed pitches are given as indeces of the chord
             if len(p) > 2:
@@ -658,7 +662,7 @@ class PCmidiR:
                 return(PCmidiR(tmp))
 
     def VLOp(self,name):
-        # operate on the pcs with a normal-ordered relational operator R({x})
+        """Apply the specified voice-leading operator to the MIDI chord."""
         op = []
         for num in re.findall("[-\d]+", name):
             op.append(int(num))
@@ -667,7 +671,7 @@ class PCmidiR:
         return(PCmidiR(selfto))
     
     def opsNameVL(self,b):
-        # given two vectors returns the name of the normal-ordered voice-leading operator R that connects them
+        """Return the name of the voice-leading operator that maps ``self`` to ``b``."""
         a = self.midi
         b = b.midi  
         diff = b-a
@@ -825,6 +829,7 @@ class PCmidiR:
             return(c)
 
 class MIDIset:
+    """Mutable MIDI pitch collection that exposes voice-leading and sequencing utilities."""
     
     def __init__(self,midi,UNI=False,ORD=False,TET=12):
         '''
@@ -866,6 +871,7 @@ class MIDIset:
         self.TET = TET
 
     def pitches(self):
+        """Return the pitch names associated with the stored MIDI numbers."""
         if isinstance(self.midi,list):
             self.midi = np.array(self.midi)
         if isinstance(self.midi[0].tolist(),int) or isinstance(self.midi[0].tolist(),float):
@@ -885,7 +891,7 @@ class MIDIset:
         •	I operation, including voice-leading preserving contextual inversion
         '''
         if not isinstance(p,list):
-            self.midi = p-self.midi+p
+            self.midi = p-self.midi
         else:
             # fixed pitches are given as indeces of the chord
             if len(p) > 2:
@@ -904,7 +910,7 @@ class MIDIset:
                     self.midi = inv + octave*12
                 
     def VLOp(self,name):
-        # operate on the pcs with a normal-ordered relational operator R({x})
+        """Apply the relational voice-leading operator encoded in ``name`` to the set."""
         op = []
         for num in re.findall("[-\d]+", name):
             op.append(int(num))
@@ -971,6 +977,7 @@ class MIDIset:
         return((np.roll(self.midi,-1)-self.midi)%self.TET)
     
     def exchange(self,voices=[0,1],mode=[1,-1]):
+        """Return a new ordering where the specified voices exchange octaves."""
         tmp = np.sort(self.midi.tolist())
         tmp[voices] += np.array(mode)*self.TET
         return(tmp.tolist())
@@ -1141,22 +1148,37 @@ class MIDIset:
 
 
 class PCSrow:
+    """Helper for manipulating twelve-tone rows via T/I/R/M/Q transformations."""
 #     Helper class for 12-tone rows operations (T,I,R,M,Q)
+
     def __init__(self,pcs,TET=12):
+        """Initialise the row with a pitch-class sequence modulo ``TET``."""
         self.pcs = np.array(pcs)%TET
         self.TET = TET
+
     def normalOrder(self):
+        """Return the row in normal order with the first element at zero."""
         self.pcs -= self.pcs[0]
         return(PCSrow(self.pcs%self.TET))
+
     def intervals(self):
+        """Return the ordered interval sequence (LIS vector) of the row."""
         return((np.roll(self.pcs,-1)-self.pcs)%self.TET)
+
     def T(self,t=0):
+        """Return the transposed row by ``t`` semitones."""
         return(PCSrow((self.pcs+t)%self.TET,TET=self.TET))
+
     def I(self,pivot=0):
+        """Return the inversion of the row around the given ``pivot``."""
         return(PCSrow((pivot-self.pcs)%self.TET,TET=self.TET))
+
     def R(self,t=0):
+        """Return the retrograde of the row optionally followed by ``t`` transposition."""
         return(PCSrow(self.pcs[::-1]).T(t))
+
     def Q(self):
+        """Return the Morris and Starr Q-transformation of the row."""
         lisv = PCSrow(self.pcs).intervals()
         lisvQ = np.roll(lisv,np.where(lisv==6)[0][1]-np.where(lisv==6)[0][0])
         Qrow = [0]
@@ -1164,10 +1186,14 @@ class PCSrow:
             Qrow.append((Qrow[-1]+n)%self.TET)
         Qrow.pop()
         return(PCSrow(Qrow))
+
     def M(self):
+        """Return the multiplication of the row by five modulo ``TET``."""
         return(PCSrow((self.pcs*5)%self.TET//1,TET=self.TET))
+
     def constellation(self):
 #         Following Morris and Starr
+        """Return the Morris and Starr constellation table for the row."""
         reference = []
         entry = ['P',str(self.pcs),str(self.I().pcs),str(self.M().I().pcs),str(self.M().pcs)]
         reference.append(entry)
@@ -1180,8 +1206,10 @@ class PCSrow:
         reference.append(entry)
         star = pd.DataFrame(reference,columns=['','P','I','IM','M'])
         return(star)
+
     def star(self):
 #         star of the row in prime form
+        """Return the star (P, I, R, Q, M) of the row."""
         reference = []
         entry = ['P',(self.pcs)]
         reference.append(entry)
@@ -1197,6 +1225,7 @@ class PCSrow:
         return(star)
 
 class RHYTHMSeq:
+    """Representation of rhythmic sequences with set-theoretic and transformational utilities."""
 
     def __init__(self,rseq,REF='e',ORD=False):
         '''
@@ -1361,6 +1390,7 @@ class RHYTHMSeq:
                             .replace('(','').replace(')','').replace('\n','').replace('[','').replace(']',''))
         
     def binrep(self):
+        """Return the binary onset representation of the rhythm sequence."""
         bincoding = []
         for n in self.reduce2GCD()[:-1]:
             for i in range(int(n)):
@@ -1371,8 +1401,7 @@ class RHYTHMSeq:
         return(bincoding)
     
     def rhythm_canon(self,pattern=None):
-        # Finds the minimal length sequence of a rhythmic pattern and its augmentations that 
-        # produces a homogeneous pulse of beats (from ANDRANIK TANGIAN, Perspective of New Music, 2003)
+        """Generate a rhythmic canon following Tangian's algorithm for homogeneous pulses."""
         
         # seed pattern
         if pattern == None:
@@ -1447,8 +1476,10 @@ class RHYTHMSeq:
         return
 
 class musicntwrk:
+    """Facade providing access to MUSIC𝄞NTWRK dictionaries, networks, harmony, timbre, and sonification tools."""
     
     def __init__(self,TET=12):
+        """Initialise the facade with the desired equal-temperament divisor."""
         self.TET = TET
         
     def dictionary(self,space=None,N=None,Nc=None,order=None,row=None,a=None,prob=None,REF=None,scorefil=None,music21=None,
